import java.awt.Color;
import java.awt.Graphics2D;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

public class Game {

	int[][] currentJewels;
	int[][] nextJewels;
	int[] stateOfNextJewels;
	int selectedX = -1;
	int selectedY = -1;
	int points = 0;
	int lastMovePoints = 0;

	public Game() {
		do {
			this.currentJewels = new int[8][8];
			for (int i = 0; i < 8; i++) {
				for (int j = 0; j < 8; j++) {
					this.currentJewels[i][j] = (int) (Math.random() * 6) + 1;
				}
			}
			this.nextJewels = new int[8][8];
			for (int i = 0; i < 8; i++) {
				for (int j = 0; j < 8; j++) {
					this.nextJewels[i][j] = (int) (Math.random() * 6) + 1;
				}
			}
			this.stateOfNextJewels = new int[8];
			while (countPointsAndMove() != 0) {
			}
		} while (!this.movesExist());
	}

	public Game(String filePath) {
		try {
			FileInputStream stream = new FileInputStream(new File(filePath));
			InputStreamReader reader = new InputStreamReader(stream);
			for (int i = 0; i < 8; i++) {
				for (int j = 0; j < 8; j++) {
					this.currentJewels[j][i] = Integer.parseInt(String.valueOf(reader.read()));
				}
				reader.read();
				reader.read();
			}
		} catch (NumberFormatException | IOException e) {
			e.printStackTrace();
		}
	}

	public boolean movesExist() {
		for (int i = 1; i <= 6; i++) {
			for (int j = 0; j < 8; j++) {
				for (int k = 0; k < 8; k++) {
					if (this.isTileThisColor(j, k, i)) {
						if (this.isTileThisColor(j + 1, k, i)) {
							if (this.isTileThisColor(j - 1, k - 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j - 1, k + 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j + 2, k - 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j + 2, k + 1, i)) {
								return true;
							}
						}
						if (this.isTileThisColor(j + 2, k, i)) {
							if (this.isTileThisColor(j + 1, k - 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j + 1, k + 1, i)) {
								return true;
							}
						}
						if (this.isTileThisColor(j, k + 1, i)) {
							if (this.isTileThisColor(j - 1, k - 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j + 1, k - 1, i)) {
								return true;
							}
							if (this.isTileThisColor(j - 1, k + 2, i)) {
								return true;
							}
							if (this.isTileThisColor(j + 1, k + 2, i)) {
								return true;
							}
						}
					}
				}
			}
		}
		return false;
	}

	boolean isTileThisColor(int x, int y, int color) {
		if (x < 0 || x >= 8 || y < 0 || y >= 8) {
			return false;
		}
		return this.currentJewels[x][y] == color;
	}

	public void draw(Graphics2D g) {
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, 600, 750);
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				g.setColor(colorFromInt(this.currentJewels[j][i]));
				g.fillRect(j * 75 + 25, (i + 1) * 75 + 25, 25, 25);
			}
		}
		if (this.selectedX != -1) {
			g.setColor(Color.CYAN);
			g.fillRect(this.selectedX * 75 + 30, (this.selectedY + 1) * 75 + 30, 15, 15);
		}
		g.setColor(Color.WHITE);
		g.setFont(g.getFont().deriveFont(20f));
		g.drawString(this.points + " points" + (this.lastMovePoints == 0 ? "" : " +" + this.lastMovePoints), 20, 715);
		g.setColor(Color.darkGray);
		g.fillRect(0, 0, 600, 75);
		for (int i = 0; i < 8; i++) {
			g.setColor(colorFromInt(this.nextJewels[i][this.stateOfNextJewels[i]]));
			g.fillRect(i * 75 + 25, 25, 25, 25);
		}
	}

	static Color colorFromInt(int colorCode) {
		switch (colorCode) {
		case 1:
			return Color.RED;
		case 2:
			return Color.GREEN;
		case 3:
			return Color.BLUE;
		case 4:
			return Color.WHITE;
		case 5:
			return Color.YELLOW;
		case 6:
			return Color.MAGENTA;
		default:
			return Color.BLACK;
		}
	}

	int countPointsAndMove() {
		int[][] pointMap = new int[8][8];
		for (int i = 1; i <= 6; i++) {
			for (int j = 0; j < 8; j++) {
				for (int k = 0; k < 8; k++) {
					if (this.isTileThisColor(j, k, i) && this.isTileThisColor(j + 1, k, i)
							&& this.isTileThisColor(j + 2, k, i)) {
						pointMap[j][k]++;
						pointMap[j + 1][k]++;
						pointMap[j + 2][k]++;
					}
					if (this.isTileThisColor(j, k, i) && this.isTileThisColor(j, k + 1, i)
							&& this.isTileThisColor(j, k + 2, i)) {
						pointMap[j][k]++;
						pointMap[j][k + 1]++;
						pointMap[j][k + 2]++;
					}
				}
			}
		}
		int totalPoints = 0;
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (pointMap[j][i] != 0) {
					this.currentJewels[j][i] = 0;
					totalPoints += pointMap[j][i];
				}
				System.out.print(pointMap[j][i]);
			}
			System.out.print("\n");
		}
		System.out.print("\n");

		if (totalPoints == 0) {
			return 0;
		}
		for (int i = 0; i < 8; i++) {
			int numberOfFreeSpaces = 0;
			for (int j = 7; j >= 0; j--) {
				if (this.currentJewels[i][j] == 0) {
					numberOfFreeSpaces++;
				} else {
					this.currentJewels[i][j + numberOfFreeSpaces] = this.currentJewels[i][j];
				}
			}
			while (numberOfFreeSpaces != 0) {
				this.currentJewels[i][numberOfFreeSpaces - 1] = this.getNextJewel(i);
				numberOfFreeSpaces--;
			}
		}
		return totalPoints;
	}

	public void handleClick(int x, int y) {
		if (y >= 75 && y < 675) {
			x = x / 75;
			y = y / 75 - 1;
			if (this.selectedX == -1) {
				this.selectedX = x;
				this.selectedY = y;
			} else {
				if ((x == this.selectedX - 1 && y == this.selectedY) || x == this.selectedX + 1 && y == this.selectedY
						|| y == this.selectedY - 1 && x == this.selectedX
						|| y == this.selectedY + 1 && x == this.selectedX) {
					int temp = this.currentJewels[x][y];
					this.currentJewels[x][y] = this.currentJewels[this.selectedX][this.selectedY];
					this.currentJewels[this.selectedX][this.selectedY] = temp;
					int pointsGained = this.countPointsAndMove();
					if (pointsGained == 0) {
						this.currentJewels[this.selectedX][this.selectedY] = this.currentJewels[x][y];
						this.currentJewels[x][y] = temp;
					} else {
						int morePoints = this.countPointsAndMove();
						while (morePoints != 0) {
							pointsGained += morePoints;
							morePoints = this.countPointsAndMove();
						}
						this.points += pointsGained;
						this.lastMovePoints = pointsGained;
					}
					this.selectedX = -1;
					this.selectedY = -1;
					if (!this.movesExist()) {
						Main.noMoreMoves(this.points);
					}
				} else {
					this.selectedX = x;
					this.selectedY = y;
				}
			}
		}
	}

	int getNextJewel(int column) {
		int toReturn = this.nextJewels[column][this.stateOfNextJewels[column]];
		this.stateOfNextJewels[column]++;
		if (this.stateOfNextJewels[column] >= this.nextJewels[column].length) {
			this.stateOfNextJewels[column] = 0;
		}
		return toReturn;
	}
}
