import java.util.List;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;

public class Game {

	int[][] currentJewels;

	public Game(String filePath) {
		try {
			List<String> lines = Files.readAllLines(Paths.get(filePath));
			this.currentJewels = new int[8][8];
			for (int i = 0; i < 8; i++) {
				for (int j = 0; j < 8; j++) {
					this.currentJewels[j][i] = Integer.parseInt(String.valueOf(lines.get(i + 1).charAt(j)));
				}
			}
		} catch (NumberFormatException | IOException e) {
			e.printStackTrace();
		}
	}

	public List<Move> getAvailableMoves() {
		List<Move> moves = new ArrayList<Move>();
		for (int i = 1; i <= 6; i++) {
			for (int j = 0; j < 8; j++) {
				for (int k = 0; k < 8; k++) {
					if (this.isTileThisColor(j, k, i)) {
						if (this.isTileThisColor(j + 1, k, i)) {
							if (this.isTileThisColor(j - 1, k - 1, i)) {
								moves.add(new Move(j - 1, k - 1, j - 1, k));
							}
							if (this.isTileThisColor(j - 1, k + 1, i)) {
								moves.add(new Move(j - 1, k + 1, j - 1, k));
							}
							if (this.isTileThisColor(j + 2, k - 1, i)) {
								moves.add(new Move(j + 2, k - 1, j + 2, k));
							}
							if (this.isTileThisColor(j + 2, k + 1, i)) {
								moves.add(new Move(j + 2, k + 1, j + 2, k));
							}
						}
						if (this.isTileThisColor(j + 2, k, i)) {
							if (this.isTileThisColor(j + 1, k - 1, i)) {
								moves.add(new Move(j + 1, k - 1, j + 1, k));
							}
							if (this.isTileThisColor(j + 1, k + 1, i)) {
								moves.add(new Move(j + 1, k + 1, j + 1, k));
							}
						}
						if (this.isTileThisColor(j, k + 1, i)) {
							if (this.isTileThisColor(j - 1, k - 1, i)) {
								moves.add(new Move(j - 1, k - 1, j, k - 1));
							}
							if (this.isTileThisColor(j + 1, k - 1, i)) {
								moves.add(new Move(j + 1, k - 1, j, k - 1));
							}
							if (this.isTileThisColor(j - 1, k + 2, i)) {
								moves.add(new Move(j - 1, k + 2, j, k + 2));
							}
							if (this.isTileThisColor(j + 1, k + 2, i)) {
								moves.add(new Move(j + 1, k + 2, j, k + 2));
							}
						}
						if (this.isTileThisColor(j, k + 2, i)) {
							if (this.isTileThisColor(j - 1, k + 1, i)) {
								moves.add(new Move(j - 1, k + 1, j, k + 1));
							}
							if (this.isTileThisColor(j + 1, k + 1, i)) {
								moves.add(new Move(j + 1, k + 1, j, k + 1));
							}
						}
					}
				}
			}
		}
		return moves;
	}

	boolean isTileThisColor(int x, int y, int color) {
		if (x < 0 || x >= 8 || y < 0 || y >= 8) {
			return false;
		}
		return this.currentJewels[x][y] == color;
	}

	public Move getBestMove() {
		List<Move> moves = this.getAvailableMoves();
		int bestMovePoints = -1;
		int bestMoveIndex = -1;
		for (int i = 0; i < moves.size(); i++) {
			this.applyMove(moves.get(i));
			int points = this.countPoints();
			this.applyMove(moves.get(i));
			if (points > bestMovePoints) {
				bestMovePoints = points;
				bestMoveIndex = i;
			}
		}
		return moves.get(bestMoveIndex);
	}

	void applyMove(Move move) {
		int temp = this.currentJewels[move.x1][move.y1];
		this.currentJewels[move.x1][move.y1] = this.currentJewels[move.x2][move.y2];
		this.currentJewels[move.x2][move.y2] = temp;
	}
	
	int countPoints() {
		int[][] pointMap = new int[8][8];
		for (int i = 1; i <= 6; i++) {
			for (int j = 0; j < 8; j++) {
				for (int k = 0; k < 8; k++) {
					if (this.isTileThisColor(j, k, i) && this.isTileThisColor(j + 1, k, i)
							&& this.isTileThisColor(j + 2, k, i)) {
						pointMap[j][k]++;
						pointMap[j + 1][k]++;
						pointMap[j + 2][k]++;
					}
					if (this.isTileThisColor(j, k, i) && this.isTileThisColor(j, k + 1, i)
							&& this.isTileThisColor(j, k + 2, i)) {
						pointMap[j][k]++;
						pointMap[j][k + 1]++;
						pointMap[j][k + 2]++;
					}
				}
			}
		}
		int totalPoints = 0;
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (pointMap[j][i] != 0) {
					this.currentJewels[j][i] = 0;
					totalPoints += pointMap[j][i];
				}
			}
		}
		return totalPoints;
	}
}
